@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2005-2006 by Olivier Goffart <ogoffart at kde.org>
+   Copyright (C) 2005-2009 by Olivier Goffart <ogoffart at kde.org>
 
 
    This program is free software; you can redistribute it and/or modify
@@ -18,6 +18,7 @@
 
  */
 
+#include "knotify.h"
 
 // KDE headers
 #include <kapplication.h>
@@ -28,7 +29,6 @@
 
 #include <config-runtime.h>
 
-#include "knotify.h"
 #include "knotifyconfig.h"
 #include "notifybysound.h"
 #include "notifybypopup.h"
@@ -82,6 +82,7 @@ void KNotify::addPlugin( KNotifyPlugin * p )
 void KNotify::reconfigure()
 {
 	KGlobal::config()->reparseConfiguration();
+	KNotifyConfig::reparseConfiguration();
 	loadConfig();
 }
 
@@ -109,16 +110,17 @@ void KNotify::closeNotification(int id)
 	delete e;
 }
 
-int KNotify::event( const QString & event, const QString & appname, const ContextList & contexts, const QString & text, const QPixmap & pixmap, const QStringList & actions, WId winId )
+int KNotify::event( const QString & event, const QString & appname, const ContextList & contexts, const QString & title, const QString & text, const KNotifyImage & image, const QStringList & actions, WId winId )
 {
 	m_counter++;
 	Event *e=new Event(appname , contexts , event );
 	e->id = m_counter;
 	e->ref = 1;
 
+	e->config.title=title;
 	e->config.text=text;
 	e->config.actions=actions;
-	e->config.pix=pixmap;
+	e->config.image=image;
 	e->config.winId=(WId)winId;
 	
 	m_notifications.insert(m_counter,e);
@@ -135,15 +137,16 @@ int KNotify::event( const QString & event, const QString & appname, const Contex
 	return m_counter;
 }
 
-void KNotify::update(int id, const QString &text, const QPixmap& pixmap,  const QStringList& actions)
+void KNotify::update(int id, const QString &title, const QString &text, const KNotifyImage& image,  const QStringList& actions)
 {
 	if(!m_notifications.contains(id))
 		return;
 
 	Event *e=m_notifications[id];
 	
+	e->config.title=title;
 	e->config.text=text;
-	e->config.pix = pixmap;
+	e->config.image = image;
 	e->config.actions = actions;
 	
 	foreach(KNotifyPlugin *p, m_plugins)
@@ -204,14 +207,14 @@ void KNotifyAdaptor::closeNotification(int id)
 }
 
 int KNotifyAdaptor::event(const QString &event, const QString &fromApp, const QVariantList& contexts,
-						   const QString &text, const QByteArray& image,  const QStringList& actions , qlonglong winId)
+						const QString &title, const QString &text, const QByteArray& image,  const QStringList& actions , qlonglong winId)
 //						  const QDBusMessage & , int _return )
 								  
 {
 	/* I'm not sure this is the right way to read a a(ss) type,  but it seems to work */
 	ContextList contextlist;
 	QString context_key;
-	foreach( QVariant v , contexts)
+	foreach( const QVariant &v , contexts)
 	{
 		/* this code doesn't work
 		QVariantList vl=v.toList();
@@ -228,17 +231,14 @@ int KNotifyAdaptor::event(const QString &event, const QString &fromApp, const QV
 			contextlist << qMakePair(context_key , s);
 	}
 	
-	QPixmap pixmap;
-	QDataStream in(image);
-	in >> pixmap;
-	return static_cast<KNotify *>(parent())->event(event, fromApp, contextlist, text, pixmap, actions, WId(winId));
+	return static_cast<KNotify *>(parent())->event(event, fromApp, contextlist, title, text, image, actions, WId(winId));
 }
 
 void KNotifyAdaptor::reemit(int id, const QVariantList& contexts)
 {
 	ContextList contextlist;
 	QString context_key;
-	foreach( QVariant v , contexts)
+	foreach( const QVariant &v , contexts)
 	{
 		QString s=v.toString();
 		if(context_key.isEmpty())
@@ -250,11 +250,9 @@ void KNotifyAdaptor::reemit(int id, const QVariantList& contexts)
 }
 
 
-void KNotifyAdaptor::update(int id, const QString &text, const QByteArray& image,  const QStringList& actions )
+void KNotifyAdaptor::update(int id, const QString &title, const QString &text, const QByteArray& image,  const QStringList& actions )
 {
-	QPixmap pixmap;
-	pixmap.loadFromData(image);
-	static_cast<KNotify *>(parent())->update(id, text, pixmap, actions);
+	static_cast<KNotify *>(parent())->update(id, title, text, image, actions);
 }
 
 #include "knotify.moc"
